@@ -15,11 +15,20 @@
 
 /*
    Modified by Timothy Mann, 1996
-   Last modified on Wed Aug  5 15:01:17 PDT 1998 by mann
+   Last modified on Tue May  1 20:32:35 PDT 2001 by mann
 */
 
+/*#define KBDEBUG 1*/
+/*#define JOYDEBUG 1*/
+#define KP_JOYSTICK 1         /* emulate joystick with keypad */
+/*#define KPNUM_JOYSTICK 1*/  /* emulate joystick with keypad + NumLock */
+#define SHIFT_F1_IS_F13 1     /* use if X reports Shift+F1..F8 as F13..F20 */
+/*#define SHIFT_F1_IS_F11 1*/ /* use if X reports Shift+F1..F10 as F11..F20 */
+
 #include "z80.h"
 #include "trs.h"
+#include <X11/keysym.h>
+#include <X11/X.h>
 
 #define TK(a, b) (((a)<<4)+(b))
 #define TK_ADDR(tk) (((tk) >> 4)&0xf)
@@ -92,14 +101,23 @@
 #define TK_F3           TK(7, 6)  /* M4 only */
 #define TK_Unused       TK(7, 7)
 
+/* Fake keycodes with special meanings */
 #define TK_NULL                 TK(8, 0)
-
 #define TK_Neutral              TK(8, 1)
 #define TK_ForceShift           TK(8, 2)
 #define TK_ForceNoShift         TK(8, 3)
 #define TK_ForceShiftPersistent TK(8, 4)
-
 #define TK_AllKeysUp            TK(8, 5)
+#define TK_Joystick             TK(10,  0)
+#define TK_North                TK(10,  1)
+#define TK_Northeast            TK(10,  9)
+#define TK_East                 TK(10,  8)
+#define TK_Southeast            TK(10, 10)
+#define TK_South                TK(10,  2)
+#define TK_Southwest            TK(10,  5)
+#define TK_West                 TK(10,  4)
+#define TK_Northwest            TK(10,  6)
+#define TK_Fire                 TK(10, 16)
 
 typedef struct
 {
@@ -456,8 +474,8 @@ KeyTable function_key_table[] = {
 /* 0xff52   XK_Up          */    { TK_Up, TK_Neutral },
 /* 0xff53   XK_Right       */    { TK_Right, TK_Neutral },
 /* 0xff54   XK_Down        */    { TK_Down, TK_Neutral },
-/* 0xff55   XK_Prior, XK_Page_Up  */  { TK_NULL, TK_Neutral },
-/* 0xff56   XK_Next, XK_Page_Down */  { TK_NULL, TK_Neutral },
+/* 0xff55   XK_Prior, XK_Page_Up  */  { TK_LeftShift, TK_Neutral },
+/* 0xff56   XK_Next, XK_Page_Down */  { TK_RightShift, TK_Neutral },
 /* 0xff57   XK_End         */    { TK_Unused, TK_Neutral },
 /* 0xff58   XK_Begin       */    { TK_NULL, TK_Neutral },
 /* 0xff59                  */    { TK_NULL, TK_Neutral },
@@ -520,16 +538,29 @@ KeyTable function_key_table[] = {
 /* 0xff92   XK_KP_F2       */    { TK_F2, TK_Neutral },
 /* 0xff93   XK_KP_F3       */    { TK_F3, TK_Neutral },
 /* 0xff94   XK_KP_F4       */    { TK_CapsLock, TK_Neutral },
-/* 0xff95   XK_KP_Home     */    { TK_RightShift, TK_Neutral },
+#if KP_JOYSTICK
+/* 0xff95   XK_KP_Home     */    { TK_Northwest, TK_Neutral },
+/* 0xff96   XK_KP_Left     */    { TK_West, TK_Neutral },
+/* 0xff97   XK_KP_Up       */    { TK_North, TK_Neutral },
+/* 0xff98   XK_KP_Right    */    { TK_East, TK_Neutral },
+/* 0xff99   XK_KP_Down     */    { TK_South, TK_Neutral },
+/* 0xff9a   XK_KP_Prior, XK_KP_Page_Up  */ { TK_Northeast, TK_Neutral },
+/* 0xff9b   XK_KP_Next, XK_KP_Page_Down */ { TK_Southeast, TK_Neutral },
+/* 0xff9c   XK_KP_End      */    { TK_Southwest, TK_Neutral },
+/* 0xff9d   XK_KP_Begin    */    { TK_Fire, TK_Neutral },
+/* 0xff9e   XK_KP_Insert   */    { TK_Fire, TK_Neutral },
+#else
+/* 0xff95   XK_KP_Home     */    { TK_Clear, TK_Neutral },
 /* 0xff96   XK_KP_Left     */    { TK_Left, TK_Neutral },
 /* 0xff97   XK_KP_Up       */    { TK_Up, TK_Neutral },
 /* 0xff98   XK_KP_Right    */    { TK_Right, TK_Neutral },
 /* 0xff99   XK_KP_Down     */    { TK_Down, TK_Neutral },
-/* 0xff9a   XK_KP_Prior, XK_KP_Page_Up  */ { TK_NULL, TK_Neutral },
-/* 0xff9b   XK_KP_Next, XK_KP_Page_Down */ { TK_NULL, TK_Neutral },
+/* 0xff9a   XK_KP_Prior, XK_KP_Page_Up  */ { TK_LeftShift, TK_Neutral },
+/* 0xff9b   XK_KP_Next, XK_KP_Page_Down */ { TK_RightShift, TK_Neutral },
 /* 0xff9c   XK_KP_End      */    { TK_Unused, TK_Neutral },
 /* 0xff9d   XK_KP_Begin    */    { TK_NULL, TK_Neutral },
 /* 0xff9e   XK_KP_Insert   */    { TK_Underscore, TK_Neutral },
+#endif
 /* 0xff9f   XK_KP_Delete   */    { TK_Left, TK_Neutral },
 /* 0xffa0                  */    { TK_NULL, TK_Neutral },
 /* 0xffa1                  */    { TK_NULL, TK_Neutral },
@@ -547,6 +578,18 @@ KeyTable function_key_table[] = {
 /* 0xffad   XK_KP_Subtract */    { TK_Minus, TK_Neutral },
 /* 0xffae   XK_KP_Decimal  */    { TK_Period, TK_Neutral },
 /* 0xffaf   XK_KP_Divide   */    { TK_Slash, TK_Neutral },
+#if KPNUM_JOYSTICK
+/* 0xffb0   XK_KP_0        */    { TK_Fire, TK_Neutral },
+/* 0xffb1   XK_KP_1        */    { TK_Southwest, TK_Neutral },
+/* 0xffb2   XK_KP_2        */    { TK_South, TK_Neutral },
+/* 0xffb3   XK_KP_3        */    { TK_Southeast, TK_Neutral },
+/* 0xffb4   XK_KP_4        */    { TK_West, TK_Neutral },
+/* 0xffb5   XK_KP_5        */    { TK_Fire, TK_Neutral },
+/* 0xffb6   XK_KP_6        */    { TK_East, TK_Neutral },
+/* 0xffb7   XK_KP_7        */    { TK_Northwest, TK_Neutral },
+/* 0xffb8   XK_KP_8        */    { TK_North, TK_Neutral },
+/* 0xffb9   XK_KP_9        */    { TK_Northeast, TK_Neutral },
+#else
 /* 0xffb0   XK_KP_0        */    { TK_0, TK_Neutral },
 /* 0xffb1   XK_KP_1        */    { TK_1, TK_Neutral },
 /* 0xffb2   XK_KP_2        */    { TK_2, TK_Neutral },
@@ -557,6 +600,7 @@ KeyTable function_key_table[] = {
 /* 0xffb7   XK_KP_7        */    { TK_7, TK_Neutral },
 /* 0xffb8   XK_KP_8        */    { TK_8, TK_Neutral },
 /* 0xffb9   XK_KP_9        */    { TK_9, TK_Neutral },
+#endif
 /* 0xffba                  */    { TK_NULL, TK_Neutral },
 /* 0xffbb                  */    { TK_NULL, TK_Neutral },
 /* 0xffbc                  */    { TK_NULL, TK_Neutral },
@@ -571,7 +615,12 @@ KeyTable function_key_table[] = {
 /* 0xffc5   XK_F8          */    { TK_NULL, TK_Neutral },
 /* 0xffc6   XK_F9          */    { TK_NULL, TK_Neutral },
 /* 0xffc7   XK_F10         */    { TK_NULL, TK_Neutral },
-/* In XFree86, XK_F11 to XK_F20 are produced for the shifted F1 to F10 keys */
+/* In some versions of XFree86, XK_F11 to XK_F20 are produced for the
+   shifted F1 to F10 keys, in some XK_F13 to XK_F20 are produced for
+   the shifted F1 to F8 keys, and in some the keysyms are not altered.
+   Arrgh.
+*/
+#if SHIFT_F1_IS_F11
 /* 0xffc8   XK_F11         */    { TK_F1, TK_Neutral },
 /* 0xffc9   XK_F12         */    { TK_F2, TK_Neutral },
 /* 0xffca   XK_F13         */    { TK_F3, TK_Neutral },
@@ -580,6 +629,25 @@ KeyTable function_key_table[] = {
 /* 0xffcd   XK_F16         */    { TK_0, TK_Neutral },
 /* 0xffce   XK_F17         */    { TK_NULL, TK_Neutral },
 /* 0xffcf   XK_F18         */    { TK_NULL, TK_Neutral },
+#elif SHIFT_F1_IS_F13
+/* 0xffc8   XK_F11         */    { TK_NULL, TK_Neutral },
+/* 0xffc9   XK_F12         */    { TK_NULL, TK_Neutral },
+/* 0xffca   XK_F13         */    { TK_F1, TK_Neutral },
+/* 0xffcb   XK_F14         */    { TK_F2, TK_Neutral },
+/* 0xffcc   XK_F15         */    { TK_F3, TK_Neutral },
+/* 0xffcd   XK_F16         */    { TK_CapsLock, TK_Neutral },
+/* 0xffce   XK_F17         */    { TK_AtSign, TK_Neutral },
+/* 0xffcf   XK_F18         */    { TK_0, TK_Neutral },
+#else
+/* 0xffc8   XK_F11         */    { TK_NULL, TK_Neutral },
+/* 0xffc9   XK_F12         */    { TK_NULL, TK_Neutral },
+/* 0xffca   XK_F13         */    { TK_NULL, TK_Neutral },
+/* 0xffcb   XK_F14         */    { TK_NULL, TK_Neutral },
+/* 0xffcc   XK_F15         */    { TK_NULL, TK_Neutral },
+/* 0xffcd   XK_F16         */    { TK_NULL, TK_Neutral },
+/* 0xffce   XK_F17         */    { TK_NULL, TK_Neutral },
+/* 0xffcf   XK_F18         */    { TK_NULL, TK_Neutral },
+#endif
 /* 0xffd0   XK_F19         */    { TK_NULL, TK_Neutral },
 /* 0xffd1   XK_F20         */    { TK_NULL, TK_Neutral },
 /* 0xffd2   XK_F21         */    { TK_NULL, TK_Neutral },
@@ -632,45 +700,86 @@ KeyTable function_key_table[] = {
 
 static int keystate[8] = { 0, };
 static int force_shift = TK_Neutral;
+static int joystate = 0;
 
 /* Avoid changing state too fast so keystrokes aren't lost. */
-static int stretch = 0;
-int stretch_amount = 16, stretch_poll = 1, stretch_heartbeat = 1;
+#define STRETCH_AMOUNT 4000
+static tstate_t key_stretch_timeout;
+int stretch_amount = STRETCH_AMOUNT;
 
 void trs_kb_reset()
 {
-    /* Be responsive right after a reset */
-    stretch = 0;
+  key_stretch_timeout = z80_state.t_count;
 }
 
+int key_heartbeat = 0;
 void trs_kb_heartbeat()
 {
-    /* Be responsive if we are polled rarely */
-    if (stretch > 0) {
-        stretch -= stretch_heartbeat;
-    }
+  /* Don't hold keys in queue too long */
+  key_heartbeat++;
 }
 
-void trs_xlate_keycode(int keycode)
+void trs_kb_bracket(int shifted)
+{
+  /* Set the shift state for the emulation of the "[ {", "\ |", 
+     "] }", "^ ~", and "_ DEL" keys.  Some Model 4 keyboard drivers
+     decode these with [ shifted and { unshifted, etc., while most
+     other keyboard drivers either ignore them or decode them with
+     [ unshifted and { shifted.  We default to the latter.  Note that
+     these keys didn't exist on real machines anyway.
+  */
+  int i;
+  for (i=0x5b; i<=0x5f; i++) {
+    ascii_key_table[i].shift_action =
+      shifted ? TK_ForceShift : TK_ForceNoShift;
+  }
+  for (i=0x7b; i<=0x7f; i++) {
+    ascii_key_table[i].shift_action =
+      shifted ? TK_ForceNoShift : TK_ForceShift;
+  }
+}
+
+/* Emulate joystick with the keypad */
+int trs_emulate_joystick(int key_down, int bit_action)
+{
+  if (bit_action < TK_Joystick) return 0;
+  if (key_down) {
+    joystate |= (bit_action & 0x1f);
+  } else {
+    joystate &= ~(bit_action & 0x1f);
+  }
+  return 1;
+}
+
+int trs_joystick_in()
+{
+#if JOYDEBUG
+  debug("joy %02x ", joystate);
+#endif
+  return ~joystate;
+}
+
+void trs_xlate_keysym(int keysym)
 {
     int key_down;
     KeyTable* kt;
     static int shift_action = TK_Neutral;
 
-    if (keycode == 0x10000) {
+    if (keysym == 0x10000) {
 	/* force all keys up */
 	queue_key(TK_AllKeysUp);
 	shift_action = TK_Neutral;
 	return;
     }
 
-    key_down = (keycode & 0x10000) == 0;
-    if (keycode & 0xff00) {
-      kt = &function_key_table[keycode & 0xff];
+    key_down = (keysym & 0x10000) == 0;
+    if (keysym & 0xff00) {
+      kt = &function_key_table[keysym & 0xff];
     } else {
-      kt = &ascii_key_table[keycode & 0xff];
+      kt = &ascii_key_table[keysym & 0xff];
     }
     if (kt->bit_action == TK_NULL) return;
+    if (trs_emulate_joystick(key_down, kt->bit_action)) return;
 
     if (key_down) {
       if (shift_action != TK_ForceShiftPersistent &&
@@ -694,7 +803,7 @@ static void change_keystate(int action)
     int key_down;
     int i;
 #ifdef KBDEBUG
-    fprintf(stderr, "change_keystate: action 0x%x\n", action);
+    debug("change_keystate: action 0x%x\n", action);
 #endif
 
     switch (action) {
@@ -760,30 +869,64 @@ int trs_kb_mem_read(int address)
 {
     int key = -1;
     int i, wait;
+    static int recursion = 0;
+    static int timesseen;
 
-    stretch -= stretch_poll;
-    if (stretch < 0) {
-	/* Check if we are in the system keyboard driver, called from
-           the wait-for-input routine.  The test below works on both
-           Model I and III and is insensitive to what keyboard driver
-           is being used, as long as it is called through the
-           wait-for-key routine at ROM address 0x0049 and has not
-           pushed too much on the stack yet when it first reads from
-           the key matrix.  The search is needed (at least) for
-           NEWDOS80, which pushes 2 extra bytes on the stack.
-	*/
-	wait = 0;
-	for (i=0; i<=4; i+=2) {
-	    if (mem_read_word(REG_SP + 2 + i) == 0x4015) {
-		wait = mem_read_word(REG_SP + 10 + i) == 0x004c;
-		break;
-	    }
+    /* Prevent endless recursive calls to this routine (by mem_read_word
+       below) if REG_SP happens to point to keyboard memory. */
+    if (recursion) return 0;
+
+    /* Avoid delaying key state changes in queue for too long */
+    if (key_heartbeat > 2) {
+      do {
+	key = trs_next_key(0);
+	if (key >= 0) {
+	  change_keystate(key);
+	  timesseen = 1;
 	}
-	key = trs_next_key(wait);
-	stretch = stretch_amount;
+      } while (key >= 0);
     }
 
-    if (key >= 0) change_keystate(key);
+    /* After each key state change, impose a timeout before the next one
+       so that the Z-80 program doesn't miss any by polling too rarely,
+       and so that we don't tickle the bugs in some common TRS-80 keyboard
+       drivers that strike if two keys change simultaneously */
+    if (key_stretch_timeout - z80_state.t_count > TSTATE_T_MID) {
+
+	/* Check if we are in the system keyboard driver, called from
+	   the wait-for-input routine.  If so, and there are no
+	   keystrokes queued, and the current state has been seen by
+	   at least 16 such reads, then trs_next_key will pause the
+	   process to avoid burning host CPU needlessly.
+
+	   The test below works on both Model I and III and is
+	   insensitive to what keyboard driver is being used, as long
+	   as it is called through the wait-for-key routine at ROM
+	   address 0x0049 and has not pushed too much on the stack yet
+	   when it first reads from the key matrix.  The search is
+	   needed (at least) for NEWDOS80, which pushes 2 extra bytes
+	   on the stack.  */
+	wait = 0;
+	if (timesseen++ >= 16) {
+	  recursion = 1;
+	  for (i=0; i<=4; i+=2) {
+	    if (mem_read_word(REG_SP + 2 + i) == 0x4015) {
+	      wait = mem_read_word(REG_SP + 10 + i) == 0x004c;
+	      break;
+	    }
+	  }
+	  recursion = 0;
+	}
+	/* Get the next key */
+	key = trs_next_key(wait);
+	key_stretch_timeout = z80_state.t_count + stretch_amount;
+    }
+
+    if (key >= 0) {
+      change_keystate(key);
+      timesseen = 1;
+    }
+    key_heartbeat = 0;
     return kb_mem_value(address);
 }
 
